@@ -1,5 +1,7 @@
+import csv
 import itertools
 import re
+import typing
 from pathlib import Path
 from typing import Optional, Union
 
@@ -29,15 +31,30 @@ INTERCHANGE_FORMAT_COLUMN_ORDER = [
     "cat_name_translation",
 ]
 
-# Pretty basic schema for now; attrs could be declared more explicitly with the
-# mandatory columns etc.
 INTERCHANGE_FORMAT_STRICTYAML_SCHEMA = sy.Map(
     {
         sy.Optional("data_file"): sy.Str(),
-        "attrs": sy.MapPattern(sy.Str(), sy.Any()),
+        "attrs": sy.Map(
+            {
+                "area": sy.Str(),
+                sy.Optional("cat"): sy.Str(),
+                sy.Optional("sec_cats"): sy.Seq(sy.Str()),
+                sy.Optional("scen"): sy.Str(),
+                sy.Optional("references"): sy.Str(),
+                sy.Optional("rights"): sy.Str(),
+                sy.Optional("contact"): sy.Str(),
+                sy.Optional("title"): sy.Str(),
+                sy.Optional("comment"): sy.Str(),
+                sy.Optional("institution"): sy.Str(),
+                sy.Optional("history"): sy.Str(),
+                sy.Optional("entity_terminology"): sy.Str(),
+                sy.Optional("publication_date"): sy.Datetime(),
+            }
+        ),
         "dimensions": sy.MapPattern(sy.Str(), sy.Seq(sy.Str())),
         "time_format": sy.Str(),
         sy.Optional("additional_coordinates"): sy.MapPattern(sy.Str(), sy.Str()),
+        sy.Optional("dtypes"): sy.MapPattern(sy.Str(), sy.Str()),
     }
 )
 
@@ -67,7 +84,7 @@ def dates_to_dimension(ds: xr.Dataset, time_format: str = "%Y") -> xr.DataArray:
     return da
 
 
-def metadata_for_variable(unit: str, variable: str) -> dict:
+def metadata_for_variable(unit: str, variable: str) -> typing.Dict[str, str]:
     """Convert a primap2 unit and variable key to a metadata dict.
 
     Derives the information needed for the data variable's attrs dict from the unit
@@ -93,14 +110,17 @@ def metadata_for_variable(unit: str, variable: str) -> dict:
     {'units': 'Gg CO2 / year', 'gwp_context': 'SARGWP100', 'entity': 'Kyoto-GHG'}
 
     """
-    attrs = {"units": unit}
+    attrs = {}
+    if unit != "no unit":
+        attrs["units"] = unit
 
     regex_gwp = r"\s\(([A-Za-z0-9]*)\)$"
-    regex_variable = r"^(.*)\s\([a-zA-z0-9]*\)$"
-
     gwp = re.findall(regex_gwp, variable)
+
     if gwp:
         attrs["gwp_context"] = gwp[0]
+
+        regex_variable = r"^(.*)\s\([a-zA-z0-9]*\)$"
         entity = re.findall(regex_variable, variable)
         if not entity:
             logger.error("Can't extract entity from " + variable)
@@ -133,7 +153,9 @@ def write_interchange_format(
         Interchange format meta data dict. Default: use data.attrs .
     """
     if attrs is None:
-        attrs = data.attrs
+        attrs = data.attrs.copy()
+    else:
+        attrs = attrs.copy()
 
     # make sure filepath is a Path object
     filepath = Path(filepath)
@@ -141,7 +163,7 @@ def write_interchange_format(
     meta_file = filepath.parent / (filepath.stem + ".yaml")
 
     # write the data
-    data.to_csv(data_file, index=False, float_format="%g")
+    data.to_csv(data_file, index=False, quoting=csv.QUOTE_NONNUMERIC)
 
     attrs["data_file"] = data_file.name
 
@@ -181,21 +203,22 @@ def read_interchange_format(
         yaml = sy.load(meta_file.read(), schema=INTERCHANGE_FORMAT_STRICTYAML_SCHEMA)
         meta_data = yaml.data
 
-    if "data_file" in meta_data.keys():
-        data = pd.read_csv(filepath.parent / meta_data["data_file"])
-    else:
-        # no file information given, check for file with same name
-        datafile = filepath.parent / (filepath.stem + ".csv")
-        if datafile.exists():
-            data = pd.read_csv(datafile)
-        else:
-            raise FileNotFoundError(
-                f"Data file not specified in {filepath} and data file not found at "
-                f"{datafile}."
-            )
+    data_file = filepath.parent / meta_data.get("data_file", filepath.stem + ".csv")
+    if not data_file.exists():
+        raise FileNotFoundError(f"Data file not found at {data_file}.")
 
+    data = pd.read_csv(data_file, dtype=object)
     data.attrs = meta_data
 
+    # strictyaml parses a datetime, we only want a date
+    if "publication_date" in data.attrs["attrs"]:
+        data.attrs["attrs"]["publication_date"] = data.attrs["attrs"][
+            "publication_date"
+        ].date()
+    # already read in
+    if "data_file" in data.attrs:
+        del data.attrs["data_file"]
+
     return data
 
 
@@ -273,22 +296,26 @@ def from_interchange_format(
     units = data_xr["unit"]
     del data_xr["unit"]
 
-    # build full dimensions dict from specification with default entry "*"
+    # build full dimensions dict from specification with default from entry "*"
     entities = np.unique(data_xr[entity_col].values)
-    dimensions = {}
+    dimensions = attrs["dimensions"]
     for entity in entities:
-        if entity in attrs["dimensions"]:
-            dims = attrs["dimensions"][entity]
-        else:
-            dims = attrs["dimensions"]["*"]
-        dimensions[entity] = dims
-    attrs["dimensions"] = dimensions
+        if entity not in dimensions:
+            dimensions[entity] = dimensions["*"]
+    if "*" in dimensions:
+        del dimensions["*"]
+
+    # build full dtypes dict from specification with default float
+    dtypes = attrs.get("dtypes", {})
+    for entity in entities:
+        if entity not in dtypes:
+            dtypes[entity] = "float"
 
     # check resulting shape to estimate memory consumption
     dim_lens = {dim: len(np.unique(data_xr[dim].dropna("index"))) for dim in index_cols}
     dim_lens["time"] = len(time_cols)
     shapes = []
-    for entity, dims in attrs["dimensions"].items():
+    for entity, dims in dimensions.items():
         shapes.append([dim_lens[dim] for dim in dims if dim != "unit"])
     array_size = sum(np.product(shape) for shape in shapes)
     logger.debug(f"Expected array shapes: {shapes}, resulting in size {array_size:,}.")
@@ -310,14 +337,14 @@ def from_interchange_format(
     else:
         da = dates_to_dimension(data_xr)
     data_vars = {}
-    for entity, dims in attrs["dimensions"].items():
+    for entity, dims in dimensions.items():
         da_entity = da.loc[{entity_col: entity}]
         # we still have a full MultiIndex, so trim it to the relevant dimensions
         da_entity["index"] = da_entity.indexes["index"].droplevel(
             list(index_cols - set(dims))
         )
         # now we can safely unstack the index
-        data_vars[entity] = da_entity.unstack("index")
+        data_vars[entity] = da_entity.unstack("index").astype(dtypes[entity])
 
     data_xr = xr.Dataset(data_vars)
 
